@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,12 +41,7 @@
 #include <dbus/dbus-glib-lowlevel.h>
 #include <gnome-keyring.h>
 #include <gconf/gconf-client.h>
-#include <devkit-power-gobject/devicekit-power.h>
-
-#ifdef HAVE_POLKIT
-#include <polkit/polkit.h>
-#include <polkit-dbus/polkit-dbus.h>
-#endif
+#include <libupower-glib/upower.h>
 
 #include "egg-debug.h"
 #include "egg-console-kit.h"
@@ -61,14 +56,12 @@
 struct GpmControlPrivate
 {
 	GConfClient		*conf;
-	DkpClient		*client;
+	UpClient		*client;
 };
 
 enum {
-	REQUEST,
 	RESUME,
 	SLEEP,
-	SLEEP_FAILURE,
 	LAST_SIGNAL
 };
 
@@ -90,127 +83,6 @@ gpm_control_error_quark (void)
 	return quark;
 }
 
-/**
- * gpm_control_is_user_privileged:
- * @polkit: This polkit class instance
- * Return value: Success value.
- **/
-static gboolean
-gpm_control_is_user_privileged (GpmControl *control, const gchar *privilege)
-{
-	gboolean ret = TRUE;
-#ifdef HAVE_POLKIT
-	polkit_uint64_t mask;
-	pid_t pid;
-
-	pid = getpid ();
-	mask = polkit_check_auth (pid, privilege, NULL);
-	if (mask == 0) {
-		egg_warning ("failed to authorise for privilege %s", privilege);
-		ret = FALSE;
-	}
-#endif
-	return ret;
-}
-
-/**
- * gpm_control_check_foreground_console:
- * @manager: This class instance
- * @action: The action we want to do, e.g. "suspend"
- *
- * Checks if the difference in time between this request for an action, and
- * the last action completing is larger than the timeout set in gconf.
- *
- * Also check for the foreground console if we specified
- * --enable-checkfg on the command line. This is only needed on Debian.
- *
- * Return value: TRUE if we can perform the action.
- **/
-static gboolean
-gpm_control_check_foreground_console (GpmControl *control)
-{
-#ifdef HAVE_CHECK_FG
-	gchar *argv[] = { "check-foreground-console", NULL };
-	int retcode;
-
-	if (!g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL,
-				 NULL, NULL, &retcode, NULL)  || ! WIFEXITED (retcode)) {
-		/* if check-foreground-console could not be executed,
-		 * assume active console */
-		egg_debug ("could not execute check-foreground-console");
-		return TRUE;
-	}
-	egg_debug ("check-foreground-console returned with %i", WEXITSTATUS (retcode));
-	return WEXITSTATUS (retcode) == 0;
-#endif
-	/* no other checks failed, so return okay */
-	return TRUE;
-}
-
-/**
- * gpm_control_allowed_suspend:
- * @control: This class instance
- * @can: If we can suspend
- *
- * checks gconf to see if we are allowed to suspend this computer.
- **/
-gboolean
-gpm_control_allowed_suspend (GpmControl *control, gboolean *can, GError **error)
-{
-	gboolean conf_ok;
-	gboolean polkit_ok;
-	gboolean hardware_ok;
-	gboolean fg;
-	g_return_val_if_fail (can, FALSE);
-
-	*can = FALSE;
-
-	/* get values from DkpClient */
-	g_object_get (control->priv->client,
-		      "can-suspend", &hardware_ok,
-		      NULL);
-
-	conf_ok = gconf_client_get_bool (control->priv->conf, GPM_CONF_CAN_SUSPEND, NULL);
-	polkit_ok = gpm_control_is_user_privileged (control, "org.freedesktop.devicekit.power.suspend");
-	fg = gpm_control_check_foreground_console (control);
-	if (conf_ok && hardware_ok && polkit_ok && fg)
-		*can = TRUE;
-	egg_debug ("conf=%i, polkit=%i, fg=%i, can=%i", conf_ok, polkit_ok, fg, *can);
-	return TRUE;
-}
-
-/**
- * gpm_control_allowed_hibernate:
- * @control: This class instance
- * @can: If we can hibernate
- *
- * Checks the HAL key power_management.can_suspend_to_disk and also
- * checks gconf to see if we are allowed to hibernate this computer.
- **/
-gboolean
-gpm_control_allowed_hibernate (GpmControl *control, gboolean *can, GError **error)
-{
-	gboolean conf_ok;
-	gboolean polkit_ok;
-	gboolean hardware_ok;
-	gboolean fg;
-	g_return_val_if_fail (can, FALSE);
-
-	/* get values from DkpClient */
-	g_object_get (control->priv->client,
-		      "can-hibernate", &hardware_ok,
-		      NULL);
-
-	*can = FALSE;
-	conf_ok = gconf_client_get_bool (control->priv->conf, GPM_CONF_CAN_HIBERNATE, NULL);
-	fg = gpm_control_check_foreground_console (control);
-	polkit_ok = gpm_control_is_user_privileged (control, "org.freedesktop.devicekit.power.hibernate");
-	if (conf_ok && hardware_ok && polkit_ok && fg)
-		*can = TRUE;
-	egg_debug ("conf=%i, polkit=%i, fg=%i, can=%i", conf_ok, polkit_ok, fg, *can);
-	return TRUE;
-}
-
 /**
  * gpm_control_shutdown:
  * @control: This class instance
@@ -265,24 +137,23 @@ gboolean
 gpm_control_suspend (GpmControl *control, GError **error)
 {
 	gboolean allowed;
-	gboolean ret;
+	gboolean ret = FALSE;
 	gboolean do_lock;
 	gboolean nm_sleep;
 	gboolean lock_gnome_keyring;
 	GnomeKeyringResult keyres;
 	GpmScreensaver *screensaver;
+	guint32 throttle_cookie = 0;
 
 	screensaver = gpm_screensaver_new ();
 
-	gpm_control_allowed_suspend (control, &allowed, error);
-	if (allowed == FALSE) {
+	g_object_get (control->priv->client,
+		      "can-suspend", &allowed,
+		      NULL);
+	if (!allowed) {
 		egg_debug ("cannot suspend as not allowed from policy");
-		g_set_error (error,
-			     GPM_CONTROL_ERROR,
-			     GPM_CONTROL_ERROR_GENERAL,
-			     "Cannot suspend as not allowed from policy");
-		g_signal_emit (control, signals [SLEEP_FAILURE], 0, GPM_CONTROL_ACTION_SUSPEND);
-		return FALSE;
+		g_set_error_literal (error, GPM_CONTROL_ERROR, GPM_CONTROL_ERROR_GENERAL, "Cannot suspend");
+		goto out;
 	}
 
 	/* we should perhaps lock keyrings when sleeping #375681 */
@@ -294,8 +165,10 @@ gpm_control_suspend (GpmControl *control, GError **error)
 	}
 
 	do_lock = gpm_control_get_lock_policy (control, GPM_CONF_LOCK_ON_SUSPEND);
-	if (do_lock)
+	if (do_lock) {
+		throttle_cookie = gpm_screensaver_add_throttle (screensaver, "suspend");
 		gpm_screensaver_lock (screensaver);
+	}
 
 	nm_sleep = gconf_client_get_bool (control->priv->conf, GPM_CONF_NETWORKMANAGER_SLEEP, NULL);
 	if (nm_sleep)
@@ -305,25 +178,23 @@ gpm_control_suspend (GpmControl *control, GError **error)
 	egg_debug ("emitting sleep");
 	g_signal_emit (control, signals [SLEEP], 0, GPM_CONTROL_ACTION_SUSPEND);
 
-	ret = dkp_client_suspend (control->priv->client, error);
+	ret = up_client_suspend_sync (control->priv->client, NULL, error);
 
 	egg_debug ("emitting resume");
 	g_signal_emit (control, signals [RESUME], 0, GPM_CONTROL_ACTION_SUSPEND);
 
-	if (!ret) {
-		egg_debug ("emitting sleep-failure");
-		g_signal_emit (control, signals [SLEEP_FAILURE], 0, GPM_CONTROL_ACTION_SUSPEND);
-	}
-
-	if (do_lock)
+	if (do_lock) {
 		gpm_screensaver_poke (screensaver);
+		if (throttle_cookie)
+			gpm_screensaver_remove_throttle (screensaver, throttle_cookie);
+	}
 
 	nm_sleep = gconf_client_get_bool (control->priv->conf, GPM_CONF_NETWORKMANAGER_SLEEP, NULL);
 	if (nm_sleep)
 		gpm_networkmanager_wake ();
 
+out:
 	g_object_unref (screensaver);
-
 	return ret;
 }
 
@@ -334,25 +205,23 @@ gboolean
 gpm_control_hibernate (GpmControl *control, GError **error)
 {
 	gboolean allowed;
-	gboolean ret;
+	gboolean ret = FALSE;
 	gboolean do_lock;
 	gboolean nm_sleep;
 	gboolean lock_gnome_keyring;
 	GnomeKeyringResult keyres;
 	GpmScreensaver *screensaver;
+	guint32 throttle_cookie = 0;
 
 	screensaver = gpm_screensaver_new ();
 
-	gpm_control_allowed_hibernate (control, &allowed, error);
-
-	if (allowed == FALSE) {
+	g_object_get (control->priv->client,
+		      "can-hibernate", &allowed,
+		      NULL);
+	if (!allowed) {
 		egg_debug ("cannot hibernate as not allowed from policy");
-		g_set_error (error,
-			     GPM_CONTROL_ERROR,
-			     GPM_CONTROL_ERROR_GENERAL,
-			     "Cannot hibernate");
-		g_signal_emit (control, signals [SLEEP_FAILURE], 0, GPM_CONTROL_ACTION_HIBERNATE);
-		return FALSE;
+		g_set_error_literal (error, GPM_CONTROL_ERROR, GPM_CONTROL_ERROR_GENERAL, "Cannot hibernate");
+		goto out;
 	}
 
 	/* we should perhaps lock keyrings when sleeping #375681 */
@@ -365,8 +234,10 @@ gpm_control_hibernate (GpmControl *control, GError **error)
 	}
 
 	do_lock = gpm_control_get_lock_policy (control, GPM_CONF_LOCK_ON_HIBERNATE);
-	if (do_lock)
+	if (do_lock) {
+		throttle_cookie = gpm_screensaver_add_throttle (screensaver, "hibernate");
 		gpm_screensaver_lock (screensaver);
+	}
 
 	nm_sleep = gconf_client_get_bool (control->priv->conf, GPM_CONF_NETWORKMANAGER_SLEEP, NULL);
 	if (nm_sleep)
@@ -375,25 +246,23 @@ gpm_control_hibernate (GpmControl *control, GError **error)
 	egg_debug ("emitting sleep");
 	g_signal_emit (control, signals [SLEEP], 0, GPM_CONTROL_ACTION_HIBERNATE);
 
-	ret = dkp_client_hibernate (control->priv->client, error);
+	ret = up_client_hibernate_sync (control->priv->client, NULL, error);
 
 	egg_debug ("emitting resume");
 	g_signal_emit (control, signals [RESUME], 0, GPM_CONTROL_ACTION_HIBERNATE);
 
-	if (!ret) {
-		egg_debug ("emitting sleep-failure");
-		g_signal_emit (control, signals [SLEEP_FAILURE], 0, GPM_CONTROL_ACTION_HIBERNATE);
-	}
-
-	if (do_lock)
+	if (do_lock) {
 		gpm_screensaver_poke (screensaver);
+		if (throttle_cookie)
+			gpm_screensaver_remove_throttle (screensaver, throttle_cookie);
+	}
 
 	nm_sleep = gconf_client_get_bool (control->priv->conf, GPM_CONF_NETWORKMANAGER_SLEEP, NULL);
 	if (nm_sleep)
 		gpm_networkmanager_wake ();
 
+out:
 	g_object_unref (screensaver);
-
 	return ret;
 }
 
@@ -443,24 +312,6 @@ gpm_control_class_init (GpmControlClass *klass)
 			      NULL,
 			      g_cclosure_marshal_VOID__INT,
 			      G_TYPE_NONE, 1, G_TYPE_INT);
-	signals [SLEEP_FAILURE] =
-		g_signal_new ("sleep-failure",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmControlClass, sleep_failure),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__INT,
-			      G_TYPE_NONE, 1, G_TYPE_INT);
-	signals [REQUEST] =
-		g_signal_new ("request",
-			      G_TYPE_FROM_CLASS (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmControlClass, request),
-			      NULL,
-			      NULL,
-			      g_cclosure_marshal_VOID__STRING,
-			      G_TYPE_NONE, 1, G_TYPE_STRING);
 
 	g_type_class_add_private (klass, sizeof (GpmControlPrivate));
 }
@@ -474,7 +325,7 @@ gpm_control_init (GpmControl *control)
 {
 	control->priv = GPM_CONTROL_GET_PRIVATE (control);
 
-	control->priv->client = dkp_client_new ();
+	control->priv->client = up_client_new ();
 	control->priv->conf = gconf_client_get_default ();
 }
 
